#--------------------------------------------------------------------------------------
#
#                                     POVERTY INDICATORS
#
#--------------------------------------------------------------------------------------

# rm(list=ls(all=TRUE))
# load the library laeken
library(laeken)

# The Laeken indicators is a set of common European statistical indicators on poverty and social 
# exclusionin Belgium. 

# EU-SILC: European Union Statistics on Income and Living Conditions 
# it is an survey "aiming at collecting timely and comparable cross-sectional and longitudinal 
# multidimensional microdata on income, poverty, social exclusion and living conditions".

# load synthetic Austrian EU-SILC data
data(eusilc)
dim(eusilc)

# rb050: the personal sample weights.
# eqIncome: a slightly simplified version of the equivalized household income
# db040: the federal state in which the household is located (9 levels Burgenland, Carinthia, 
# Lower Austria, Salzburg, Styria, Tyrol, Upper Austria, Vienna and Vorarlberg).

# The variable eqIncome contains for each individual the household equivalized total net income, 
# calculated as the household total net income divided by the equivalised household size
# the equivalence scale is the modified OECD scale
# "Modified OECD equivalence scale", weights:
# 1 for the 1st adult (>14 years )(household head)
# 0.5 for other adults
# 0.3 for children(<14)

# Equivalence scales are commonly employed in income analysis to compare the wealth of households 
# of various compositions 

#--------------------------------------------------------------------
#                      Head Count Ratio (HCR)
#--------------------------------------------------------------------

# the Head Count Ratio (HCR)  is defined  as  the  proportion  of  persons  with  an
# equivalized disposable income below the at-risk-at-poverty line. 

# poverty threshold: is equal to the 60% of the national median equivalised income
#help("weightedMedian")
poverty.line<-weightedMedian(eusilc$eqIncome, eusilc$rb050)*0.6
poverty.line

# Define if an individual is poor assign 1 if his eqIncome is below the at-risk-at-poverty line
risk.pov<- ifelse(eusilc$eqIncome <= poverty.line, 1, 0)
eusilc$risk.pov<-risk.pov

#calulcate HCR as weighted mean of people that have an income below the poverty line 
HCR<-weighted.mean(risk.pov, w=eusilc$rb050)
HCR
HCR*100


# We can alternatively use the functions inside the package "laeken" to compute the HCR

# poverty threshold can be calculated with function arpt()
arpt(eusilc$eqIncome, weights = eusilc$rb050)


# the function arpr() is implemented to estimate the HCR
# the function gives also the povety threshold
arpr(eusilc$eqIncome, weights = eusilc$rb050)

# HCR federal states level (NUTS 2)
# NUTS: Nomenclature of Territorial Units for Statistics 
# it is a common statistical classification of territorial units in  order  to  allow  
# the  collection,  compilation and  dissemination  of  European  statistics  at  different  
# territorial  levels  of  the  European Union.


# In order to estimate the HCR at NUTS 2 (identified by variable db040)
# we have to calculate the weighted mean of risk.pov for each regions
# Explore the variable db040
str(eusilc$db040)
table (eusilc$db040)
#build a datafrem with name of regions and frequencie
tab_db040<-data.frame(table(eusilc$db040))
View(tab_db040)

# calcalate the HCR:
# measures the proportion of the population that is poor, i.e. the proportion of 
# a population that lives below the poverty line
HCR_NUTS2<-as.numeric(lapply(split(eusilc, eusilc$db040), function(eusilc) weighted.mean(eusilc$risk.pov, eusilc$rb050)))
HCR_AUT2<-data.frame(REGION=tab_db040$Var1,HCR_NUTS2=HCR_NUTS2)
HCR_AUT2
# We can calcuate the HCR in each regions by using the function arpr() 
# setting the breakdown argument (categorical variable!)
arpr(eusilc$eqIncome, weights = eusilc$rb050, breakdown = eusilc$db040)

# HCR with RELATIVE POVERTY LINE

# reative poverty line 
rel.povertyline<-as.numeric(lapply(split(eusilc, eusilc$db040), function(eusilc) weightedMedian(eusilc$eqIncome, eusilc$rb050)))*0.6
#everything is okay? Proof seeing if using one region I Obtain the same result!
# Burgenland <- eusilc[which(eusilc$db040 == "Burgenland"),]
# weightedMedian(Burgenland$eqIncome, Burgenland$rb050)*0.6

# add the variable at the dataframe for calculate the HCR on the basis of 
# the relative poverty line
# Firstly, order the dataset because the values in rel.povertyline are ordered on the basis
# of db040 (I prefer to do not work on the original dataset but I define an other dataframe)
eusilc2 <- eusilc[order(eusilc$db040),] 
# then repeat the relative poverty line for the number of obs in each region
eusilc2$rel.poveryline=rep(rel.povertyline, tab_db040$Freq)

# Define if an individual is poor assign 1 if his eqIncome is below the 
# relative poverty line
eusilc2$rel.risk.pov<- ifelse(eusilc2$eqIncome <= eusilc2$rel.poveryline, 1, 0)

#use the same function above to compute the HCR
HCR_rel<-as.numeric(lapply(split(eusilc2, eusilc2$db040), function(eusilc2) weighted.mean(eusilc2$rel.risk.pov, eusilc2$rb050)))
AUT_POV<-data.frame(REGION=tab_db040$Var1, 
                    ABS_POV.LINE=poverty.line,
                    HCR_abs=HCR_NUTS2,
                    REL_POV.LINE=rel.povertyline,
                    HCR_rel=HCR_rel
                    )
AUT_POV


#--------------------------------------------------------------------
#                         RELATIVE POVERTY GAP
#--------------------------------------------------------------------
# PG is  given  by  the  difference  between  the median equivalized disposable income of persons
# below the at-risk-of-poverty threshold and the  at-risk  of  poverty  threshold  itself  

# it measures the poverty intensity, since it takes into account the distance from the
# poverty line.

# build a dataset with only poors
poors<-eusilc [ (eusilc$eqIncome<poverty.line),]
#calcuate the median
poors_med<-weightedMedian(poors$eqIncome, poors$rb050)
# calculate the RPG
# =0 implies that no one in the population is below the poverty line. 
# =100 implies that everyone in the population has zero income.
PG<-(poverty.line-poors_med)/poverty.line
PG
# The RPG is implemented in the function rmpg()
rmpg("eqIncome", weights = "rb050", data = eusilc)

# relative median at-risk-of-poverty gap: federal states level (NUTS 2)
#First, i need a function that calculate the RPG
my_PG <- function(pov.line,income,w) {
   med.poors=weightedMedian(income, w) 
  (pov.line-med.poors)/pov.line
}
#Then I can use lapply to split my dataset on the basis of NUTS 2 code and apply 
# my function to each NUTS calculating the RPG at NUTS 2 level
PG_NUTS<-as.numeric(lapply(split(poors, poors$db040), function(poors) my_PG(poverty.line, poors$eqIncome, poors$rb050)))
# add the variable at the my dataframe summarising poverty indicators
AUT_POV$PG_NUTS=PG_NUTS
AUT_POV

# The relative median at-risk-of-poverty gap is implemented in the function rmpg()
rmpg(eusilc$eqIncome, weights = eusilc$rb050, breakdown = eusilc$db040)



# QUINTILE SHARE RATIO 
# The income quintile share ratio (QSR) is defined as the ratio of the sum of the equivalized 
# disposable income received by the 20% of the population with the highest equivalized disposable
# income to that received by the 20% of the population with the lowest equivalized disposable
# income
# To estimate the quintile share ratio, the function qsr() is available.
qsr("eqIncome", weights = "rb050", data = eusilc)


#--------------------------------------------------------------------------------------
#
#                                          MAPPING 
#
#--------------------------------------------------------------------------------------
# in the following we use R to plot in a map the HCR at NUTS2 level in Austria

library(maptools)
library(RColorBrewer)
library(classInt)
library(sp)
library(lattice)

# In order to plot a map we have to load the shape file corresponding to the Country of interest
# Shapefiles are so-called vector databases, describing the location of points 
# (e.g., collecting locations), polylines (e.g., roads)  andareas (or polygons, e.g., soil types,  
# countries). A shapefile actually consists of three separate files with the same name 
# but with different extensions (SHP, SHX and DBF), but they are treated as one file. 

# you can use this website to download shape files:
# http://www.diva-gis.org/datadown

# use function readShapePoly() from package maptools
# The readShapePoly() reads data from a polygon shapefile into a SpatialPolygonsDataFrame object.
# inside the folder AUT_adm I have a series of file
# the file useful for me in this case is AUT_adm1.shp
Austria=readShapePoly("C:\\Users\\franc\\OneDrive\\Documenti\\WEB PISA\\Lezioni JM 2018\\R Lab 26 Oct 2018\\AUT_adm\\AUT_adm1.shp")
# Warning meassage because the use of this function is deprecated and it is not being maintained
# but it works!

# Alterantively you can use rgdal::readOGR()
# install.packages("rgdal") #install package
library(rgdal)
# in this case I load Austria shapefile with NUTS-3 border cahnginf the name of file
# I use AUT_adm2.shp
Austria3<-rgdal::readOGR("C:\\Users\\franc\\OneDrive\\Documenti\\WEB PISA\\Lezioni JM 2018\\R Lab 26 Oct 2018\\\\AUT_adm\\AUT_adm2.shp")


# plot to see the difference of Austria and Austria3
plot(Austria)
plot (Austria3) # more detailed maps

# The I have to merge my SpatialPolygonsDataFrame object (Austria) with my data
# To do this I need a common variable
# Looking at the file AUT_adm1.dbf I can see that each Region is identified by means of a ID
# 1	Burgenland
# 2	Carinthia
# 3	Lower Austria
# 4	Upper Austria
# 5	Salzburg
# 6	Styria
# 7	Tyrol
# 8	Vorarlberg
# 9	Vienna

# So I add at my dataframe containing the HCR and the name of the regions an ID 
# for each region according to UT_adm1.dbf

HCR_AUT2$ID_1=c(1,2,3,5,6,7,4,9,8)
HCR_AUT2

AUT<- merge(Austria, HCR_AUT2, by="ID_1")
# help(spplot) (this command is in the library {sp})
# plotting data (intial maps with no custom options)
spplot(AUT, "HCR_NUTS2", main = "HCR NUTS 2")

# If you don't like the default color scheme, you can use your own with ColorBrewer, 
# which you can install with install.packages("RColorBrewer"). 
# Once loaded, you can see a list of all the color pallets that come with RColorBrewer 
# with the command: 
display.brewer.all()

# Once you've picked a palette you like, create a palette object as follows, 
# where n is the number of cuts you want to use, and name is the name of the color ramp. 
# Note that different palettes have different limits on the maximium and minimium number of cuts 
# allowed.
pal1 <- brewer.pal(n=5,name="OrRd") 
# Then you can just pass this pallet to spplot, making sure to set the number of cuts 
# to 1 minus the number of colors.
# cuts: number of levels the range of 'z' would be divided into 
spplot(AUT, "HCR_NUTS2", main = "HCR NUTS 2", col.regions = pal1, cuts = 4)

# If you don't want equally spaced cutoffs between colors, you can use the classInt 
# library to make custom cuts.
nclr <- 4
plotclr <- brewer.pal(nclr,"OrRd")
#style	chosen style: one of "fixed", "sd", "equal", "pretty", 
#                   "quantile", "kmeans", "hclust", "bclust", "fisher", or "jenks"
# intervalClosure (default "left"), allows specification of whether 
# partition intervals are closed on the left or the right 
class <- classIntervals(AUT$HCR_NUTS2, nclr, style="quantile")
#breakpoints
categ = class[["brks"]]
categ
#assing col at each class
colcode <- findColours(class, plotclr)
#round the extremities of the class
lab=round(categ,3)
#plot (adding colorkey I can change the lateral bar with labels)
spplot(AUT, "HCR_NUTS2",
       col.regions=plotclr, 
       at=c(class$brks[1], class$brks[2],class$brks[3],class$brks[4], class$brks[5]+0.01),
       colorkey = list(space="bottom",height = 1,
         labels=list(
           at = lab,
           labels = lab
         )))       

# You can also plot the HCR divide in categories according to quantiles
# define class and add the variable at the dataframe 
AUT$cutHCR <- cut(AUT$HCR_NUTS2, breaks = categ, 
                  right = F, include.lowest=T)  
#plot (in this case I set dimensions of the bar in the legend of equal lenght )
spplot(AUT, "cutHCR", col.regions=plotclr, 
       #main = "HCR AUSTRIA (NUTS 2)",
       colorkey = list(space="bottom",height = 1, labels = list(at = seq(0.5, length(categ) -0.5), 
                                                 labels = round(categ,3))))


#ALTERNATIVE METHOD WITHOUT USING spplot()
# quantile() in order to define the breakpoints

brks.qt <- quantile(AUT$HCR_NUTS2, seq(0,1,1/4), na.rm=T)
brks.qt
# choose palette and set number of colors:
palette<-"OrRd"
n<-length(brks.qt)-1
cols <- brewer.pal(n,palette)
cols
brks=brks.qt
# I can change graphical parameters to obtain a bigger map
par(mfrow=c(1,1),mar=c(0.1,0.1,0.1,0.1))

# plot the map using the function plot() and define col on the basis of HCR
# with findInterval I define the colors on the basis of HCR
plot(Austria,col=cols[findInterval(AUT$HCR_NUTS2,brks,all.inside=TRUE)]) #, main=tit)

# add legend (many parameters, you can change it on the basis as you want)
plot.window(asp=1,xlim=c(0,1),ylim=c(0,1))
#xlim, ylim: numeric vectors of length 2, giving the x and y coordinates ranges.
#asp: numeric, giving the aspect ratio y/x.
n.lev<-length(brks)-1
lower.y<-0.5 #0
lower.x<-0 # 0
size<-0.05 # size of the legend
f.type<-1
sep<-0.01
n.digits<-3
n.lev<-length(brks)-1
for (i in 1:n.lev){
  plot.x<-c(lower.x,lower.x+size,lower.x+size,lower.x)
  plot.y<-c(lower.y+i*(size+sep),lower.y+i*(size+sep),lower.y+i*(size+sep)+size,lower.y+i*(size+sep)+size)
  brks<-round(brks,n.digits)
  polygon(plot.x,plot.y,col=cols[i])
  if (i==1) leg<-paste("<",format(brks[2],n.digits,nsmall=n.digits))
  else if (i==n.lev) leg<-paste(">",format(brks[n.lev],n.digits,nsmall=n.digits))
  else leg<-paste(format(brks[i],n.digits,nsmall=n.digits),"-",format(brks[i+1],n.digits,nsmall=n.digits))
  text(lower.x+size+sep,lower.y+i*(size+sep)+sep/10,labels=leg,adj=c(0,0),cex=1,offset=1,font=f.type)
}

dev.off()